@@ -1,25 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2015-2016 Oleg Shparber
-** Copyright (C) 2013-2014 Jerzy Kozera
-** Contact: https://go.zealdocs.org/l/contact
-**
-** This file is part of Zeal.
-**
-** Zeal is free software: you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation, either version 3 of the License, or
-** (at your option) any later version.
-**
-** Zeal is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
-**
-** You should have received a copy of the GNU General Public License
-** along with Zeal. If not, see <https://www.gnu.org/licenses/>.
-**
-****************************************************************************/
+// Copyright (C) Oleg Shparber, et al. <https://zealdocs.org>
+// Copyright (C) 2013-2014 Jerzy Kozera
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "docsetregistry.h"
 
@@ -32,6 +13,7 @@
 #include <core/httpserver.h>
 
 #include <QDir>
+#include <QLoggingCategory>
 #include <QThread>
 
 #include <QtConcurrent>
@@ -41,6 +23,8 @@
 
 using namespace Zeal::Registry;
 
+static Q_LOGGING_CATEGORY(log, "zeal.registry.docsetregistry")
+
 void MergeQueryResults(QList<SearchResult> &finalResult, const QList<SearchResult> &partial)
 {
     finalResult << partial;
@@ -82,10 +66,11 @@ void DocsetRegistry::setStoragePath(const QString &path)
         return;
     }
 
-    m_storagePath = path;
-
-    unloadAllDocsets();
-    addDocsetsFromFolder(path);
+    QMetaObject::invokeMethod(this, [this, path](){
+        unloadAllDocsets();
+        addDocsetsFromFolder(path);
+        m_storagePath = path;
+    });
 }
 
 bool DocsetRegistry::isFuzzySearchEnabled() const
@@ -131,8 +116,8 @@ void DocsetRegistry::loadDocset(const QString &path)
     Docset *docset = f.get();
     // TODO: Emit error
     if (!docset->isValid()) {
-        qWarning("Could not load docset from '%s'. Reinstall the docset.",
-                 qPrintable(docset->path()));
+        qCWarning(log, "Could not load docset '%s' from '%s'. Reinstall the docset.",
+                  qPrintable(docset->name()), qPrintable(docset->path()));
         delete docset;
         return;
     }
@@ -147,8 +132,8 @@ void DocsetRegistry::loadDocset(const QString &path)
     // Setup HTTP mount.
     QUrl url = Core::Application::instance()->httpServer()->mount(name, docset->documentPath());
     if (url.isEmpty()) {
-        qWarning("Could not enable docset from '%s'. Reinstall the docset.",
-                 qPrintable(docset->path()));
+        qCWarning(log, "Could not enable docset '%s' from '%s'. Reinstall the docset.",
+                  qPrintable(docset->name()), qPrintable(docset->path()));
         delete docset;
         return;
     }
